/*
 * Copyright (C) 2024 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __STA_H__
#define __STA_H__

#include "common.h"
#include "libpowerfip.h"

/* global handlers */
void pwrfip_var_handler(struct pwrfip_node *node,
                struct pwrfip_var *var, struct pwrfip_event *evt);
void pwrfip_msg_recv_handler(struct pwrfip_node *node,
                struct pwrfip_msg *msg, struct pwrfip_event *evt);
void pwrfip_msg_send_handler(struct pwrfip_node *node,
                struct pwrfip_msg *msg, struct pwrfip_event *evt);
void pwrfip_ba_state_handler(struct pwrfip_node *node,
                struct pwrfip_event *evt);
void pwrfip_ba_sync_handler(struct pwrfip_node *node,
    struct pwrfip_var *var, struct pwrfip_event *evt);
void pwrfip_error_handler(struct pwrfip_node *node,
                enum pwrfip_error_code code);
void pwrfip_reset_handler(struct pwrfip_node *node);

/* specific handlers */
void pwrfip_msg_recv_spec_handler(struct pwrfip_node *node,
                struct pwrfip_msg *msg, struct pwrfip_event *evt);
void pwrfip_msg_send_spec_handler(struct pwrfip_node *node,
                struct pwrfip_msg *msg, struct pwrfip_event *evt);
void print_node_infos(struct pwrfip_node *node);

/* uni-te management */
#define UNITE_RQ_READ_OBJECT 0x36
#define UNITE_CR_READ_OBJECT 0x66
void unite_rx_treat(struct pwrfip_node *node, struct pwrfip_msg *rx_msg,
                struct pwrfip_event *event);
void unite_tx_treat(struct pwrfip_node *node, struct pwrfip_msg *tx_msg,
                struct pwrfip_event *event);

#define MAX_STA_COUNT 2
#define STA_VAR_COUNT 1
static struct pwrfip_var_cfg var_cfg[MAX_STA_COUNT][STA_VAR_COUNT] = {
    /* station 1 (xway2.1) */
    {
        /* [0x0001] (prod) */
        {
            .type = PWRFIP_VAR_TYPE_PROD,
            .id = 0x0001,
            .prod.payload_bsz = 0,
            .prod.flags = \
                /* enable aper msg request */
                PWRFIP_VAR_FLAGS_APER_MSG_REQ,
            .prod.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
    },
    /* station 2 (xway2.2) */
    {
        /* [0x0002] (prod) */
        {
            .type = PWRFIP_VAR_TYPE_PROD,
            .id = 0x0002,
            .prod.payload_bsz = 0,
            .prod.flags = \
                /* enable aper msg request */
                PWRFIP_VAR_FLAGS_APER_MSG_REQ,
            .prod.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
    },
};

#define STA_MSG_COUNT 2
static struct pwrfip_msg_cfg msg_cfg[MAX_STA_COUNT][MAX_MSG_COUNT] = {
    /* station 1 (xway2.1) */
    {
        /* msg to produce [src=0x090102, dst=0x090202] */
        {
            .type = PWRFIP_MSG_TYPE_SEND,
            .tx.channel = PWRFIP_MSG_TX_CH_APER,
            .tx.ack_mode = PWRFIP_MSG_TX_ACK_MODE_SDA,
            /* local data link layer address: 0x090102 */
            .hdr.src.seg = 2,
            .hdr.src.lsap = 0x0901, /* ssap */
            /* remote data link layer address: 0x090202 */
            .hdr.dst.seg = 2,
            .hdr.dst.lsap = 0x0902, /* dsap (=target) */
            .pwrfip_msg_handler = pwrfip_msg_send_spec_handler,
        },
        /* msg to receive [src=0x090202, dst=0x090102] */
        {
            .type = PWRFIP_MSG_TYPE_RECV,
            /* remote data link layer address: 0x090202 */
            .hdr.src.seg = 2,
            .hdr.src.lsap = 0x0902, /* ssap */
            /* local data link layer address: 0x090102 */
            .hdr.dst.seg = 2,
            .hdr.dst.lsap = 0x0901, /* dsap (=target) */
            .pwrfip_msg_handler = pwrfip_msg_recv_spec_handler,
        },
    },
    /* station 2 (xway2.2) */
    {
        /* msg to produce [src=0x090202, dst=0x090102] */
        {
            .type = PWRFIP_MSG_TYPE_SEND,
            .tx.channel = PWRFIP_MSG_TX_CH_APER,
            .tx.ack_mode = PWRFIP_MSG_TX_ACK_MODE_SDA,
            /* local data link layer address: 0x090202 */
            .hdr.src.seg = 2,
            .hdr.src.lsap = 0x0902, /* ssap */
            /* remote data link layer address: 0x090102 */
            .hdr.dst.seg = 2,
            .hdr.dst.lsap = 0x0901, /* dsap (=target) */
            .pwrfip_msg_handler = pwrfip_msg_send_spec_handler,
        },
        /* msg to receive [src=0x090102, dst=0x090202] */
        {
            .type = PWRFIP_MSG_TYPE_RECV,
            /* remote data link layer address: 0x090102 */
            .hdr.src.seg = 2,
            .hdr.src.lsap = 0x0901, /* ssap */
            /* local data link layer address: 0x090202 */
            .hdr.dst.seg = 2,
            .hdr.dst.lsap = 0x0902, /* dsap (=target) */
            .pwrfip_msg_handler = pwrfip_msg_recv_spec_handler,
        },
    },
};

/* ba periodic requests list [for periodic mcycle wind] */
#define TST_BA_PER_REQ1_COUNT 36
static struct pwrfip_ba_request tst_ba_per_req1[TST_BA_PER_REQ1_COUNT] = {
    /* periodic variables */
    /* -> process variables */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0000, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0001, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0002, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0003, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0004, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0005, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0006, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0007, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1400, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1401, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1402, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1403, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1404, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1405, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1406, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1407, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1408, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1409, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140b, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ2_COUNT 29
static struct pwrfip_ba_request tst_ba_per_req2[TST_BA_PER_REQ2_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0008, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0009, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000f, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x140f, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1410, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ3_COUNT 36
static struct pwrfip_ba_request tst_ba_per_req3[TST_BA_PER_REQ3_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0010, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0011, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0012, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0013, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0014, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0015, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0016, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0017, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1411, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1412, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1413, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1414, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1415, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1416, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1417, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1418, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1419, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141c, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ4_COUNT 29
static struct pwrfip_ba_request tst_ba_per_req4[TST_BA_PER_REQ4_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0018, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0019, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001f, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x141f, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1420, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1421, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ5_COUNT 36
static struct pwrfip_ba_request tst_ba_per_req5[TST_BA_PER_REQ5_COUNT] = {
    /* periodic variables */
    /* -> process variables */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0000, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0001, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0002, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0003, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0004, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0005, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0006, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0007, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1422, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1423, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1424, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1425, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1426, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1427, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1428, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1429, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142d, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ6_COUNT 29
static struct pwrfip_ba_request tst_ba_per_req6[TST_BA_PER_REQ6_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0008, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0009, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x000f, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x142f, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1430, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1431, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1432, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ7_COUNT 37
static struct pwrfip_ba_request tst_ba_per_req7[TST_BA_PER_REQ7_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0010, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0011, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0012, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0013, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0014, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0015, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0016, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0017, },
    /* -> presence table */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1433, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1434, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1435, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1436, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1437, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1438, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x1439, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x143f, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};
#define TST_BA_PER_REQ8_COUNT 27
static struct pwrfip_ba_request tst_ba_per_req8[TST_BA_PER_REQ8_COUNT] = {
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0018, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x0019, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001a, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001b, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001c, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001d, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001e, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x001f, },
    /* -> sm-mps variables */
    { .type = PWRFIP_BA_ID_DAT, .id = 0x9008, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x9002, },
    { .type = PWRFIP_BA_ID_DAT, .id = 0x9003, },
    /* periodic messages */
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d00, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d01, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d02, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d03, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d04, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d05, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d06, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d07, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d08, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d09, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0a, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0b, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0c, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0d, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0e, },
    { .type = PWRFIP_BA_ID_MSG, .id = 0x0d0f, },
};

#define BA_MCYCLE_WIND_COUNT 24
static struct pwrfip_ba_wind_cfg ba_wind_cfg[BA_MCYCLE_WIND_COUNT] = {
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ1_COUNT,
        .per.req_list = &tst_ba_per_req1[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 9000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 10000,
        .wait.is_silent = 0, /* fill window with padding frames */
        .wait.is_ext_resync = 0, /* internal resynchronization */
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ2_COUNT,
        .per.req_list = &tst_ba_per_req2[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 19000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 20000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ3_COUNT,
        .per.req_list = &tst_ba_per_req3[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 29000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 30000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ4_COUNT,
        .per.req_list = &tst_ba_per_req4[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 39000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 40000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ5_COUNT,
        .per.req_list = &tst_ba_per_req5[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 49000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 50000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ6_COUNT,
        .per.req_list = &tst_ba_per_req6[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 59000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 60000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ7_COUNT,
        .per.req_list = &tst_ba_per_req7[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 69000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 70000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
    {
        .type = PWRFIP_BA_WIND_PER,
        .per.req_cnt = TST_BA_PER_REQ8_COUNT,
        .per.req_list = &tst_ba_per_req8[0],
    },
    {
        .type = PWRFIP_BA_WIND_APER_MSG,
        .aper_msg.end_ustime = 79000,
    },
    {
        .type = PWRFIP_BA_WIND_WAIT,
        .wait.end_ustime = 80000,
        .wait.is_silent = 0,
        .wait.is_ext_resync = 0,
    },
};

static struct pwrfip_ba_mcycle_cfg ba_mcycle_cfg = {
    .wind_cnt = BA_MCYCLE_WIND_COUNT,
    .wind_list = &ba_wind_cfg[0],
};


#define PWRFIP_BITRATE_RANGE (_PWRFIP_BITRATE_MAX - _PWRFIP_BITRATE_MIN)

/* turn around time (default) in us, according to the bitrate */
static const uint32_t tr_ustime[PWRFIP_BITRATE_RANGE] = {
    424, /* 424us@31.25Kbps */
    30, /* 30us@1Mbps */
    30, /* 30us@2.5Mbps */
    32, /* 32us@5Mbps */
    32, /* 32us@12.5Mbps */
    32, /* 32us@25Mbps */
};

/* silence time (default) in us, according to the bitrate */
static const uint32_t ts_ustime[PWRFIP_BITRATE_RANGE] = {
    4096, /* 4096us@31.25Kbps */
    150, /* 150us@1Mbps */
    96, /* 96us@2.5Mbps */
    92, /* 92us@5Mbps */
    92, /* 92us@12.5Mbps */
    92, /* 92us@25Mbps */
};

static const char *pwrfip_node_state_str[_PWRFIP_NODE_STATE_MAX] = {
    /* PWRFIP_NODE_STATE_INITIAL */
    "initial",
    /* PWRFIP_NODE_STATE_LOADED */
    "loaded",
    /* PWRFIP_NODE_STATE_READY */
    "ready",
    /* PWRFIP_NODE_STATE_RUNNING */
    "running",
};

const char *pwrfip_ba_state_str[_PWRFIP_BA_STATE_MAX] = {
    /* PWRFIP_BA_STATE_INITIAL */
    "initial",
    /* PWRFIP_BA_STATE_READY */
    "ready",
    /* PWRFIP_BA_STATE_STARTING */
    "starting",
    /* PWRFIP_BA_STATE_IDLE */
    "idle",
    /* PWRFIP_BA_STATE_RUNNING */
    "running",
};

static const char *pwrfip_node_op_str[_PWRFIP_NODE_OP_MAX] = {
    /* _PWRFIP_NODE_OP_UNKNOWN */
    "unknown",
    /* PWRFIP_NODE_OP_WAIT_RX_RP_FRM */
    "rx rp frame",
    /* PWRFIP_NODE_OP_WAIT_TX_RP_FRM */
    "tx rp frame",
    /* PWRFIP_NODE_OP_WAIT_RX_ID_FRM */
    "rx id frame",
    /* PWRFIP_NODE_OP_WAIT_TX_ID_FRM */
    "tx id frame",
};

static const char *pwrfip_ba_wind_str[_PWRFIP_BA_WIND_TYPE_MAX] = {
    /* _PWRFIP_BA_WIND_TYPE_NONE */
    "none",
    /* PWRFIP_BA_WIND_PER */
    "periodic",
    /* PWRFIP_BA_WIND_APER_VAR */
    "aper. var",
    /* PWRFIP_BA_WIND_APER_MSG */
    "aper. msg",
    /* PWRFIP_BA_WIND_WAIT */
    "wait",
};

static const char *pwrfip_var_err_str[_PWRFIP_VAR_ERR_MAX] = {
    /* _PWRFIP_VAR_OK */
    "no error",
    /* PWRFIP_VAR_ID_UNKNOWN */
    "unknown var id",
    /* PWRFIP_VAR_NOT_PRODUCING */
    "not a producing var",
    /* PWRFIP_VAR_NOT_CONSUMING */
    "not a consuming var",
    /* PWRFIP_VAR_TX_APER_CH_UNKNOWN */
    "unknown tx aper channel",
    /* PWRFIP_VAR_PDU_INCONSISTENT */
    "inconsistent var pdu",
    /* PWRFIP_VAR_LEN_TOO_LONG */
    "var length too long",
    /* PWRFIP_VAR_LEN_TOO_SHORT */
    "var length too short",
    /* PWRFIP_VAR_NEVER_RECEIVED */
    "var never received",
    /* PWRFIP_VAR_TX_APER_FIFO_EMPTY */
    "tx aper queue is empty",
    /* PWRFIP_VAR_TX_APER_FIFO_FULL */
    "tx aper queue is full",
    /* PWRFIP_VAR_NOT_MEANING */
    "payload not significant",
    /* PWRFIP_VAR_NOT_REFRESH */
    "payload not refreshed",
    /* PWRFIP_VAR_NOT_PROMPT */
    "payload not prompt",
    /* PWRFIP_VAR_BAD_PROMPT_PER */
    "bad reading frequency (promptness)",
    /* PWRFIP_VAR_BAD_REFRESH_PER */
    "bad writing frequency (refreshment)"
};

static const char *pwrfip_msg_err_str[_PWRFIP_MSG_ERR_MAX] = {
    /* _PWRFIP_MSG_OK */
    "no error",
    /* PWRFIP_MSG_TX_CH_UNKNOWN */
    "unknown message tx channel number",
    /* PWRFIP_MSG_TYPE_BAD */
    "bad message type",
    /* PWRFIP_MSG_LEN_BAD */
    "bad message length",
    /* PWRFIP_MSG_RX_FIFO_EMPTY */
    "rx message queue is empty",
    /* PWRFIP_MSG_TX_FIFO_EMPTY */
    "tx message queue is empty",
    /* PWRFIP_MSG_TX_FIFO_FULL */
    "tx message queue is full",
    /* PWRFIP_MSG_TX_TMO */
    "msg never sent and timeout expired",
    /* PWRFIP_MSG_TX_ACK_TMO */
    "msg sent but ack never received",
    /* PWRFIP_MSG_TX_ACK_NACK */
    "msg sent but ack is negative"
};

#endif
